#ifndef LMO_COLLECTIVES_H_
#define LMO_COLLECTIVES_H_

#include "lmo.h"
#include "cpm_collectives.h"

/*!
 * \defgroup lmo_collectives LMO-based collective operations
 * \{
 */
#ifdef __cplusplus
extern "C" {
#endif

/*!
 * \brief The global instance of the LMO model for use in the optimized scatter/gather.
 * \details Must be initialized by \ref LMO_initialize and destroyed by \ref LMO_finalize on all processes. 
 */
extern LMO_model* LMO_model_instance;

/*!
 * \brief Initializes the instances of the LMO model (\ref LMO_model_instance) on all processes in the communicatior.
 * \param comm MPI communicator 
 * \param model LMO model (significant only at root)
 */
int LMO_initialize(MPI_Comm comm, LMO_model* model);

/*!
 * \brief Destroys the instances of the LMO model (\ref LMO_model_instance) on all processes in the communicatior.
 */
int LMO_finalize(MPI_Comm comm);

/*!
 * \brief Split flat-tree MPI_Scatter.
 * \details \ref LMO_model_instance must be initialized.
 */
int LMO_Scatter_split_flat(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype, 
	int root, MPI_Comm comm);

/*!
 * \brief Split flat-tree MPI_Gather.
 * \details \ref LMO_model_instance must be initialized.
 */
int LMO_Gather_split_flat(void* sendbuf, int sendcount, MPI_Datatype sendtype,
	void* recvbuf, int recvcount, MPI_Datatype recvtype,
	int root, MPI_Comm comm);

#ifdef __cplusplus
}
#endif
/*!
 * \}
 */

#endif /*LMO_COLLECTIVES_H_*/
