#ifndef OBSERVATION_H_
#define OBSERVATION_H_

#include "cpm_models.h"
#include "mpi.h"
#include "mpib.h"
#include <gsl/gsl_interp.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct LinInterp_model {
	/*! Predictor */
	CPM_predictor predictor;
	/*! Number of nodes */
	int n;
	/*! Numbre of measurement points per link*/
	int num_points;
	gsl_interp** workspace;
	double** x;
	double** y;
	//gsl_interp_accel *accel;
} LinInterp_model;

LinInterp_model* LinInterp_alloc(int size, int num_points);
void LinInterp_free(LinInterp_model* model);
void LinInterp_estimate(MPI_Comm comm, MPIB_precision precision, MPIB_msgset msgset, int parallel,
		 LinInterp_model** _model);
void LinInterp_read(FILE* stream, LinInterp_model** model);
void LinInterp_write(FILE* stream, const LinInterp_model* model);
double LinInterp_predict_p2p(void* _this, int i, int j, int M);
#ifdef __cplusplus
}
#endif
#endif /*OBSERVATION_H_*/
