#ifndef LMO_UTILITIES_H_
#define LMO_UTILITIES_H_

#include <Rinternals.h>

/*!
 * \defgroup lmo_utilities LMO utilities
 * \{
 */

/*! \f$ C_n^3 \f$ */
#define LMO_C3(n) (n) * ((n) - 1) * ((n) - 2) / 6

/*!
 * The index of the \f$ (i, j, k) \f$ element in the array of \f$ C_n^3 \f$ elements, \f$ i < j < k < n \f$.
 * \f$ \displaystyle\frac{C_{n - 1}^2 + C_{n - i}^2}{2} i + \frac{(n - i + 1) + (n - j)}{2} (j - i) + (k - j - 1) \f$.
 */
#define LMO_IJK2INDEX(n, i, j, k) (2 * (n) - (i) - 1) * ((i) - 1) * (i) / 4 + (2 * (n) - (i) - (j) + 1) * ((j) - (i)) / 2 + (k) - (j) - 1

/*! Initializes R */
int LMO_initR();

/*! Finalizes R */
void LMO_endR();

/*! Gets an element given by the name from R list */
SEXP LMO_getListElement(SEXP list, char *str);

/*!
 * \}
 */

#endif /*LMO_UTILITIES_H_*/
